REPOS_YUM = "yum/static/"
HARMCHECKSUM = "yum/harm_checksum/%s/"
MISSINGFILE = "yum/not_found/%s/"
BADURL = "yum/badurl/"
BADGPG = "yum/badgpg/"
AUTHBASIC = "yum/auth_basic/"
PARTIAL = "yum/partial/"

AUTH_USER = "admin"
AUTH_PASS = "secret"

REPO_YUM_01_PATH = REPOS_YUM+"01/"
REPO_YUM_01_REPODATA = REPO_YUM_01_PATH+"repodata/"
REPO_YUM_01_REPOMD = REPO_YUM_01_REPODATA+"repomd.xml"
REPO_YUM_01_PATH_VAR = REPOS_YUM+"$version/"
REPO_YUM_01_VARSUB_LIST = [("version", "01")]

REPO_YUM_02_PATH = REPOS_YUM+"02/"

REPO_YUM_03_PATH = REPOS_YUM+"03/"

REPO_YUM_04_PATH = REPOS_YUM+"04/"

METALINK_DIR = "yum/static/metalink/"
METALINK_GOOD_01 = METALINK_DIR+"good_01.xml"
METALINK_GOOD_02 = METALINK_DIR+"good_02.xml"
METALINK_BADFILENAME = METALINK_DIR+"badfilename.xml"
METALINK_BADCHECKSUM = METALINK_DIR+"badchecksum.xml"
METALINK_NOURLS = METALINK_DIR+"nourls.xml"
METALINK_BADFIRSTURL = METALINK_DIR+"badfirsturl.xml"
METALINK_BADFIRSTHOST = METALINK_DIR+"badfirsthost.xml"
METALINK_FIRSTURLHASCORRUPTEDFILES = METALINK_DIR+"firsturlhascorruptedfiles.xml"
METALINK_VARSUB = METALINK_DIR+"varsub.xml"
METALINK_VARSUB_LIST = [("version", "01")]
METALINK_WITH_ALTERNATES = METALINK_DIR+"metalink_with_alternates.xml"
METALINK_MIRRORS_01 = METALINK_DIR+"mirrors_01.xml"

MIRRORLIST_DIR = "yum/static/mirrorlist/"
MIRRORLIST_GOOD_01 = MIRRORLIST_DIR+"good_01"
MIRRORLIST_GOOD_02 = MIRRORLIST_DIR+"good_02"
MIRRORLIST_NOURLS = MIRRORLIST_DIR+"nourls"
MIRRORLIST_BADFIRSTURL = MIRRORLIST_DIR+"badfirsturl"
MIRRORLIST_FIRSTURLHASCORRUPTEDFILES = MIRRORLIST_DIR+"firsturlhascorruptedfiles"
MIRRORLIST_VARED = MIRRORLIST_DIR+"$var"
MIRRORLIST_VARED_LIST = [("var", "varsub"), ("version", "01")]
MIRRORLIST_VARSUB = MIRRORLIST_DIR+"varsub"
MIRRORLIST_VARSUB_LIST = [("version", "01")]

# PACKAGE_<repo_id>_<package_id>
REPO_YUM_01_PACKAGES = REPO_YUM_01_PATH
PACKAGE_01_01 = "filesystem-2.4.44-1.fc16.i686.rpm"
PACKAGE_01_01_SHA256 = "3a2d4370e617056c9f76ec9ad1a7c699db8da90d3cc860943035ca0fda8136f4"

REPO_YUM_03_PACKAGES = REPO_YUM_03_PATH
PACKAGE_03_01 = "Archer-3.4.5-6.x86_64.rpm"

REPO_YUM_04_PACKAGES = REPO_YUM_04_PATH
PACKAGE_04_01 = "empty-0-0.src.rpm"
